"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var lodash_1 = __importDefault(require("lodash"));
var request_promise_1 = __importDefault(require("request-promise"));
// tslint:disable-next-line: no-var-requires
var CFError = require('cf-errors');
var LIMIT_PER_PAGE = 500;
var MAX_PAGES = 20;
var ApiVersions = {
    V1: 'rest/api/1.0/'
};
var _cleanEncodedFilePath = function (filepath) {
    var prefix = './';
    var hasPrefix = filepath.indexOf(prefix) === 0;
    var path = hasPrefix ? filepath.substring(prefix.length) : filepath;
    return encodeURIComponent(path);
};
var BitbucketServer = /** @class */ (function () {
    function BitbucketServer(opts) {
        if (!opts.apiURL) {
            throw new CFError('Cannot create bitbucket-server provider without api url');
        }
        this.baseUrl = opts.apiURL;
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = this.baseUrl + "/";
        }
        this.timeout = opts.timeout || 10000;
        if (opts.type === 'oauth2') {
            this.authenticationHeader = { Authorization: "Bearer " + opts.password };
        }
        else {
            this.authenticationHeader = { Authorization: "Basic " + Buffer.from(opts.username + ":" + opts.password).toString('base64') };
        }
    }
    BitbucketServer.prototype.performAPICall = function (opts) {
        return __awaiter(this, void 0, void 0, function () {
            var method, requestHeaders, requestOptions;
            return __generator(this, function (_a) {
                method = opts.method || 'GET';
                requestHeaders = lodash_1["default"].merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
                requestOptions = {
                    method: method,
                    url: "" + this.baseUrl + opts.apiVersion + opts.api,
                    qs: opts.qs || {},
                    headers: requestHeaders,
                    json: opts.json,
                    timeout: this.timeout,
                    form: opts.data,
                    resolveWithFullResponse: true
                };
                return [2 /*return*/, request_promise_1["default"](requestOptions)];
            });
        });
    };
    BitbucketServer.prototype.fetchRawFile = function (opts) {
        return __awaiter(this, void 0, void 0, function () {
            var lines, start, page, isLastPage, res;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        lines = [];
                        start = 0;
                        page = 0;
                        isLastPage = false;
                        _a.label = 1;
                    case 1:
                        if (!(!isLastPage && page < MAX_PAGES)) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.performAPICall({
                                apiVersion: ApiVersions.V1,
                                api: "projects/" + opts.owner + "/repos/" + opts.repo + "/browse/" + _cleanEncodedFilePath(opts.path),
                                json: true,
                                qs: {
                                    at: opts.ref,
                                    raw: 'true',
                                    start: '' + start,
                                    limit: '' + LIMIT_PER_PAGE
                                }
                            })];
                    case 2:
                        res = _a.sent();
                        if (res.statusCode >= 400) {
                            throw new CFError("Failed to retrieve file: " + JSON.stringify(opts) + ", status code: " + res.statusCode);
                        }
                        isLastPage = res.body.isLastPage;
                        lines.push.apply(lines, lodash_1["default"].map(res.body.lines, function (line) { return line.text; }));
                        start += LIMIT_PER_PAGE;
                        page += 1;
                        return [3 /*break*/, 1];
                    case 3: return [2 /*return*/, lines.join('\n')];
                }
            });
        });
    };
    BitbucketServer.prototype.getBranch = function (opts) {
        return __awaiter(this, void 0, void 0, function () {
            var commitMetadataPath, branch, start, page, isLastPage, res;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        commitMetadataPath = 'com.atlassian.bitbucket.server.bitbucket-branch:latest-commit-metadata';
                        start = 0;
                        page = 0;
                        isLastPage = false;
                        _a.label = 1;
                    case 1:
                        if (!(!isLastPage && page < MAX_PAGES)) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.performAPICall({
                                apiVersion: ApiVersions.V1,
                                api: "projects/" + opts.owner + "/repos/" + opts.repo + "/branches",
                                json: true,
                                qs: {
                                    filterText: opts.branch,
                                    details: 'true',
                                    start: '' + start,
                                    limit: '' + LIMIT_PER_PAGE
                                }
                            })];
                    case 2:
                        res = _a.sent();
                        if (res.statusCode >= 400) {
                            throw new CFError("Failed to retrieve branch: " + JSON.stringify(opts) + ", status code: " + res.statusCode);
                        }
                        isLastPage = res.body.isLastPage;
                        branch = lodash_1["default"].find(res.body.values, function (b) { return lodash_1["default"].isEqual(opts.branch, b.displayId ? b.displayId : lodash_1["default"].last(b.id.split('/'))); });
                        if (branch) {
                            return [3 /*break*/, 3]; // found the correct branch
                        }
                        start += LIMIT_PER_PAGE;
                        page += 1;
                        return [3 /*break*/, 1];
                    case 3:
                        if (!branch) {
                            throw new CFError("Failed to retrieve branch: " + JSON.stringify(opts) + ", branch not found");
                        }
                        return [2 /*return*/, {
                                name: branch.displayId,
                                id: branch.id,
                                commit: {
                                    sha: branch.metadata[commitMetadataPath].id,
                                    commiter_name: branch.metadata[commitMetadataPath].author.name,
                                    message: branch.metadata[commitMetadataPath].message,
                                    url: ''
                                }
                            }];
                }
            });
        });
    };
    BitbucketServer.prototype.listBranches = function (opt) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error("Method not implemented.");
            });
        });
    };
    BitbucketServer.prototype.listRepositories = function (opt) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error("Method not implemented.");
            });
        });
    };
    BitbucketServer.prototype.createRepositoryWebhook = function (opt) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error("Method not implemented.");
            });
        });
    };
    BitbucketServer.prototype.listWebhooks = function (opt) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error("Method not implemented.");
            });
        });
    };
    return BitbucketServer;
}());
exports["default"] = BitbucketServer;
//# sourceMappingURL=bitbucket-server.js.map