export declare type ProviderConstructor = new (opt: {
    password: string;
    username?: string;
    isPrivateToken?: boolean;
    refreshToken?: string;
    type: 'basic' | 'oauth2' | 'PAT';
    apiPathPrefix?: string;
    apiHost?: string;
    apiURL?: string;
    timeout?: number;
    expires?: number;
}) => Provider;
export interface Provider {
    fetchRawFile(opt: {
        owner: string;
        repo: string;
        ref: string;
        path: string;
    }): Promise<string>;
    getBranch(opt: {
        owner: string;
        repo: string;
        branch: string;
    }): Promise<Branch>;
    listBranches(opt: {
        owner: string;
        repo: string;
        limit?: number;
    }): Promise<Branch[]>;
    listRepositories(opt: {
        owner: string;
        limit?: number;
    }): Promise<Repository[]>;
    listWebhooks(opt: {
        owner: string;
        repo: string;
        limit?: number;
    }): Promise<Webhook[]>;
    createRepositoryWebhook(opt: {
        owner: string;
        repo: string;
        endpoint: string;
        secret: string;
    }): Promise<Webhook>;
}
export interface Repository {
    id: string;
    name: string;
    full_name: string;
    description: string;
    owner: string;
    url: string;
}
export interface Branch {
    name: string;
    id: string;
    commit: Commit;
}
export interface Commit {
    sha: string;
    message: string;
    commiter_name: string;
    url: string;
}
export interface Webhook {
    id: string;
    name: string;
    events: string[];
    endpoint: string;
}
